#include "server_data_interact.h"
#include <engine_engine_number_9/baza_throw.h>
#include "../str_fields.h"

namespace iu9cawebchat {
    /* Throws excetion on failure
     * Chat's HistoryId will increment after this operation, incremented
     * if adding system message, uid is ignored
     */
    int64_t insert_new_message(SqliteConnection& conn, int64_t uid, int64_t chatId,
            const std::string& text, bool isSystem) {
        int64_t chat_HistoryId_BEFORE_MSG = get_current_history_id_of_chat(conn, chatId);
        if (chat_HistoryId_BEFORE_MSG > INT64_MAX - 100)
            een9_THROW("please no");
        SqliteStatement req(conn,
            "INSERT INTO `message` (`chatId`, `previous`, `senderUserId`, `exists`, `isSystem`, `text`,"
            "`chat_IncHistoryId`) VALUES (?1, ?2, ?3, 1, ?4, ?5, ?6)",
            {{1, chatId}, {4, (int64_t)isSystem}, {6, chat_HistoryId_BEFORE_MSG + 1}}, {{5, text}});
        int64_t chat_cur_last_msg_id = get_lastMsgId_of_chat(conn, chatId);
        if (chat_cur_last_msg_id >= 0)
            sqlite_stmt_bind_int64(req, 2, chat_cur_last_msg_id);
        if (!isSystem)
            sqlite_stmt_bind_int64(req, 3, uid);
        int64_t MSG_ID = sqlite_trsess_last_insert_rowid(conn);
        sqlite_nooutput(conn, "UPDATE `chat` SET `lastMsgId` = ?1, `it_HistoryId` = ?2 WHERE `id` = ?3",
            {{1, MSG_ID}, {2, chat_HistoryId_BEFORE_MSG + 1}, {3, chatId}}, {});
        return MSG_ID;
    }

    json::JSON internalapi_sendMessage(SqliteConnection& conn, int64_t uid, const json::JSON& Sent) {
        int64_t chatId = Sent["chatId"].g().asInteger().get_int();
        int64_t my_role_here = get_role_of_user_in_chat(conn, uid, chatId);
        if (my_role_here == user_chat_role_deleted)
            een9_THROW("Unauthorized user tries to access internalapi_getChatInfo");
        if (my_role_here == user_chat_role_read_only)
            een9_THROW("read-only user can't send messages");
        const int64_t LocalHistoryId = Sent["LocalHistoryId"].g().asInteger().get_int();
        std::string text = Sent["content"]["text"].g().asString();
        if (!is_orthodox_string(text) || text.empty())
            een9_THROW("Bad input text");
        int64_t MSG_ID = insert_new_message(conn, uid, chatId, text, false);
        json::JSON Recv;
        Recv["status"] = json::JSON(0l);
        json::JSON hist_ent_response = Recv["update"][0].g();
        event_polling_fill_chat_hist_entity_response(conn, hist_ent_response, uid, chatId, LocalHistoryId);
        return Recv;
    }
}
